bl_info = {
    "name": "Random text animator",
    "description": "generate random charcters",
    "author": "businge ismail",
    "version": (0, 0, 1),
    "blender": (2, 80, 0),
    "location": "3D View > Tools",
    "warning": "make sure you have a text object named 'count' in your scene", # used for warning icon and text in addons panel
    "wiki_url": "https://www.youtube.com/c/TopChannel1on1/videos",
    "tracker_url": "",
    "category": "Development"
}


import bpy
import string
import random



from bpy.props import (StringProperty,
                       BoolProperty,
                       IntProperty,
                       FloatProperty,
                       FloatVectorProperty,
                       EnumProperty,
                       PointerProperty,
                       )
from bpy.types import (Panel,
                       Menu,
                       Operator,
                       PropertyGroup,
                       )


# ------------------------------------------------------------------------
#    Scene Properties
# ------------------------------------------------------------------------


def random_character(size=10, chars=string.ascii_uppercase + string.digits):
    return ''.join(random.choice(chars) for _ in range(size))
    
class MyProperties(PropertyGroup):

    my_bool: BoolProperty(
        name="Only random Numbers not Characters",
        description="will generate random numbers to final output",
        default = False
        )
    final_word: StringProperty(
        name = "Enter final word / number",
        description="final display value, also you should have a text object named count in your scene",
        default = "ACCESS GRANTED",
        maxlen=1024
        )
    start_word: StringProperty(
        name = "start_words",
        description="start display value",
        default = "ENTER ACCESS KEY",
        maxlen=1024
        )
        
    long_string: StringProperty(
        name = "sentence",
        description="enter long sentence",
        default = "Exloding in 5 4 3 2 1 0",
        maxlen=1024
        )
    

    start_frame: IntProperty(
        name = "start Frame",
        description="where to start",
        default = 10,
        min = 0,
        max = 10000000
        )
    end_frame: IntProperty(
        name = "End Frame",
        description="where to end",
        default = 100,
        min = 1,
        max = 10000000
        )
        

 
   
    my_string: StringProperty(
        name="Enter Your text",
        description=":",
        default="",
        maxlen=1024,
        )

  

#------------------------------------
# functionality
#------------------------------------
 

def character_animator(scene):
    start = bpy.data.scenes['Scene'].my_tool.start_frame
    end = bpy.data.scenes['Scene'].my_tool.end_frame
    
    final_text = bpy.data.scenes['Scene'].my_tool.final_word
    start_text = bpy.data.scenes['Scene'].my_tool.start_word
    long_strings = bpy.data.scenes['Scene'].my_tool.long_string
    
    animate_bool = bpy.data.scenes['Scene'].my_tool.my_bool
    
    
    t = long_strings.split()
    
    
    number_obj = bpy.data.objects['count']
#    text_obj = bpy.data.objects['random_text']
    
    current_frame = bpy.context.scene.frame_current
    
    if(current_frame < start):
        number_obj.data.body = start_text
    elif (current_frame > start ):
        if (animate_bool):
            value = current_frame * 250
            number_obj.data.body = '{:,}'.format(value)
        else:
            number_obj.data.body = random_character()
            
        if (current_frame > end):
            number_obj.data.body = final_text
            
        
        
           
    


class WM_OT_HelloWorld(Operator):
    bl_label = "Run animator"
    bl_idname = "wm.hello_world"
   

    def execute(self, context):
        scene = context.scene
        mytool = scene.my_tool
        
        bpy.app.handlers.frame_change_post.append(character_animator)
       
       
            
            
            
        # print the values to the console
        print("/////////")
        print("bool state:", mytool.my_bool)
        
        print("start:", mytool.start_frame)
        print("end:", mytool.end_frame)
        
      
        print("string value:", mytool.my_string)
       

        return {'FINISHED'}

# ------------------------------------------------------------------------
#    Menus
# ------------------------------------------------------------------------

class OBJECT_MT_CustomMenu(bpy.types.Menu):
    bl_label = "Select"
    bl_idname = "OBJECT_MT_custom_menu"

    def draw(self, context):
        layout = self.layout

        # Built-in operators
        layout.operator("object.select_all", text="Select/Deselbbect All").action = 'TOGGLE'
        layout.operator("object.select_all", text="Inverse").action = 'INVERT'
        layout.operator("object.select_random", text="Random")

# ------------------------------------------------------------------------
#    Panel in Object Mode
# ------------------------------------------------------------------------

class OBJECT_PT_CustomPanel(Panel):
    bl_label = "Text Animator"
    bl_idname = "OBJECT_PT_custom_panel"
    bl_space_type = "VIEW_3D"   
    bl_region_type = "UI"
    bl_category = "Text_animator"
    bl_context = "objectmode"   


    @classmethod
    def poll(self,context):
        return context.object is not None

    def draw(self, context):
        layout = self.layout
        scene = context.scene
        mytool = scene.my_tool

        layout.prop(mytool, "my_bool")
        
        layout.separator()
        layout.prop(mytool, "start_word")
        layout.prop(mytool, "final_word")
        
#        layout.prop(mytool, 'long_string')
        layout.separator()
     
        
        layout.prop(mytool, "start_frame")
        layout.prop(mytool, "end_frame")
        
        
       
       
        layout.operator("wm.hello_world")
        
        layout.separator()

# ------------------------------------------------------------------------
#    Registration
# ------------------------------------------------------------------------

classes = (
    MyProperties,
    WM_OT_HelloWorld,
    OBJECT_MT_CustomMenu,
    OBJECT_PT_CustomPanel
)

def register():
    from bpy.utils import register_class
    for cls in classes:
        register_class(cls)

    bpy.types.Scene.my_tool = PointerProperty(type=MyProperties)

def unregister():
    from bpy.utils import unregister_class
    for cls in reversed(classes):
        unregister_class(cls)
    del bpy.types.Scene.my_tool


if __name__ == "__main__":
    register()